// Basic keyboard layouts
//
keyboard_ROWS= {
    'uk': [ // UK 102/105-key layout
        ['OEM_8', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', 'OEM_MINUS', 'OEM_PLUS'],
        ['Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P', 'OEM_4', 'OEM_6'],
        ['A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', 'OEM_1', 'OEM_3', 'OEM_7'],
        ['OEM_5', 'Z', 'X', 'C', 'V', 'B', 'N', 'M', 'OEM_COMMA', 'OEM_PERIOD', 'OEM_2'],
        ['SPACE']
    ],
    'us': [ // US 101/104-key layout
        ['OEM_3', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', 'OEM_MINUS', 'OEM_PLUS'],
        ['Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P', 'OEM_4', 'OEM_6', 'OEM_5'],
        ['A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', 'OEM_1', 'OEM_7'],
        ['Z', 'X', 'C', 'V', 'B', 'N', 'M', 'OEM_COMMA', 'OEM_PERIOD', 'OEM_2'],
        ['SPACE']
    ]
};

// Undisplayable characters
//
keyboard_ALTCHARS= {
    ' ': '[sp]',
    '\xA0': '[nbsp]',
    '\xAD': '[shy]',
    '\u2000': '[enq]',
    '\u2001': '[emq]',
    '\u2002': '[ensp]',
    '\u2003': '[emsp]',
    '\u2004': '[3msp]',
    '\u2005': '[4msp]',
    '\u2006': '[6msp]',
    '\u2007': '[fsp]',
    '\u2008': '[psp]',
    '\u2009': '[thsp]',
    '\u200a': '[hsp]',
    '\u200b': '[zwsp]',
    '\u200c': '[zwnj]',
    '\u200d': '[zwj]',
    '\u200e': '[lrm]',
    '\u200f': '[rlm]',
    '\u202f': '[nnbsp]',
    '\ufeff': '[bom]'
};


// Utility
//
var node_NODETYPE_ELEMENT= 1;

function element_getParentByTagName(el, name) {
    while (el.parentNode.nodeType==node_NODETYPE_ELEMENT) {
        el= el.parentNode;
        if (el.tagName.toLowerCase()==name)
            return el;
    }
    return null;
}
function element_setTextContent(el, data) {
    while (el.firstChild)
        el.removeChild(el.firstChild);
    el.appendChild(document.createTextNode(data));
}
function element_getClass(el, classname) {
    if (!el.className) return false;
    if (el.className==classname) return true;
    return array_index(el.className.split(' '), classname)!=-1;
}
function element_setClass(el, classname, active) {
    var classes= el.className.split(' ');
    var ix= array_index(classes, classname);
    if (ix==-1 && active) {
        classes[classes.length]= classname;
    } else if (ix!=-1 && !active) {
        classes.splice(ix, 1);
    }
    el.className= classes.join(' ');
}
function element_getClassValue(el, classname) {
    var prefix= classname+'-';
    var classes= el.className.split(' ');
    for (var i= classes.length; i-->0;)
        if (classes[i].substring(0, prefix.length)==prefix)
            return classes[i].substring(prefix.length);
    return null;
}
function array_index(array, item) {
    for (var i= 0; i<array.length; i++)
        if (array[i]==item)
            return i;
    return -1;
}
function number_padstring(n, radix, pad) {
    var s= n.toString(radix).toUpperCase();
    while (s.length<pad) s= '0'+s;
    return s;
}


// Keyboard layout
//
function keyboard_bind() {
    while (keyboard_el.firstChild)
        keyboard_el.removeChild(keyboard_el.firstChild);
    var rows= keyboard_ROWS[keyboard_rowtype];
    for (var rowi= 0; rowi<rows.length; rowi++) { var row= rows[rowi];
        var div= document.createElement('div');
        div.id= 'row-'+rowi;
        for (var keyi= 0; keyi<row.length; keyi++) { var key= row[keyi];
            var table= document.createElement('table');
            table.id= 'key-'+key;
            table.appendChild(document.createElement('tbody'));
            table.insertRow(0);
            table.insertRow(1);
            table.rows[0].insertCell(0).className= 'mod-1';
            table.rows[0].insertCell(1).className= 'mod-3';
            table.rows[1].insertCell(0).className= 'mod-0';
            table.rows[1].insertCell(1).className= 'mod-2';
            table.rows[0].cells[0].onclick= table.rows[0].cells[1].onclick= table.rows[1].cells[0].onclick= table.rows[1].cells[1].onclick= keyboard_click;
            div.appendChild(table);
        }
        keyboard_el.appendChild(div);
    }

    keyboard_el.appendChild(document.createElement('div'));
    var output= document.createElement('input');
    output.type= 'text';
    output.id= 'keyboard-output';
    keyboard_el.lastChild.appendChild(output);
    keyboard_set('');
}

function keyboard_click() {
    var mod= parseInt(element_getClassValue(this, 'mod'));
    var key= element_getParentByTagName(this, 'table').id.substring(4);
    var layout= keyboard_LAYOUT[keyboard_state];
    var chs= layout[key];
    if (!chs) chs= '';
    var ch= chs.substring(mod, mod+1);

    if (keyboard_state!='') {
        if (ch) {
            keyboard_output(ch);
        } else {
            keyboard_output(keyboard_state);
            keyboard_output(keyboard_LAYOUT[''][key].substring(mod&1, (mod&1)+1));
        }
        keyboard_set('');
    } else if (ch) {
        if (element_getClass(this, 'dead'))
            keyboard_set(ch);
        else
            keyboard_output(ch);
    }
}

function keyboard_set(state) {
    keyboard_state= state;
    var layout= keyboard_LAYOUT[state];
    var rows= keyboard_ROWS[keyboard_rowtype];

    basetitle= '';
    if (state!='') {
        for (var rowi= 0; rowi<rows.length; rowi++) { var row= rows[rowi];
            for (var keyi= 0; keyi<row.length; keyi++) { var key= row[keyi];
                var basechs= keyboard_LAYOUT[''][key];
                if (!basechs) basechs='\n\n\n\n    ';
                for (var chi= 0; chi<4; chi++) { var basech= basechs.substring(chi, chi+1);
                    if (basech==state && basechs.substring(chi+4, chi+5)=='@') {
                        if (chi>=2)
                            basetitle= 'AltGr+'+basechs.substring(chi-2, chi-1)+',';
                        else
                            basetitle= basech+',';
    }   }   }   }   }

    for (var rowi= 0; rowi<rows.length; rowi++) { var row= rows[rowi];
        for (var keyi= 0; keyi<row.length; keyi++) { var key= row[keyi];
            var table= document.getElementById('key-'+key);
            var chs= layout[key];
            if (!chs) chs= '\n\n\n\n    ';
            var basechs= keyboard_LAYOUT[''][key];
            if (!basechs) basechs= '\n\n\n\n    ';

            for (var mod= 0; mod<4; mod++) {
                var cell= table.rows[1-(mod%2)].cells[(mod>=2)? 1:0];
                var ch= chs.substring(mod, mod+1);
                if (ch=='\n' || (mod>=2 && ch==chs.substring(mod-2, mod-1) && chs.substring(mod+2, mod+3)==chs.substring(mod+4, mod+5)))
                    ch= '';
                var isdead= state=='' && chs.substring(mod+4, mod+5)=='@';
                var isactive= state==basechs[mod] && basechs[mod+4]=='@';

                title= '';
                if (ch!='') {
                    title= basechs.substring(mod%2, mod%2+1)
                    if (mod>=2) title= 'AltGr+'+title;
                    title= basetitle+title;
                    if (isdead)
                        title= title+' (dead key)';
                    else {
                        title= title+': U+'+number_padstring(ch.charCodeAt(0), 16, 4);
                        if (keyboard_CHARNAMES[ch]) {
                            title= title+' '+keyboard_CHARNAMES[ch];
                            if (keyboard_CHARNAMES[ch].substring(0, 9)=='Combining')
                                ch= '\xA0'+ch;
                        }
                    }
                }
                cell.title= title;
                cell.style.cursor= ch!=''? 'pointer' : 'default';

                if (keyboard_ALTCHARS[ch])
                    ch= keyboard_ALTCHARS[ch];
                element_setTextContent(cell, ch);
                element_setClass(cell, 'dead', isdead);
                element_setClass(cell, 'active', isactive);
            }
        }
    }
    element_setClass(keyboard_el, 'keystate-symbols', state=='\xA4');
}

function keyboard_output(ch) {
    document.getElementById('keyboard-output').value+= ch;
};


keyboard_el= document.getElementById('keyboard');
keyboard_rowtype= element_getClassValue(keyboard_el, 'layout');
keyboard_state= '';

if (keyboard_el) keyboard_bind();
